/*
 * Copyright (C) 2022 Uniontech Software Technology Co., Ltd.
 *
 * Author:     lanxuesong<lanxuesong@uniontech.com>
 *
 * Maintainer: lanxuesong<lanxuesong@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DECORATORFILEINFO_H
#define DECORATORFILEINFO_H

#include "dfm-base/dfm_base_global.h"

#include <dfm-io/core/dfileinfo.h>

namespace dfmbase {

class DecoratorFileInfoPrivate;
class DecoratorFileInfo
{
public:
    explicit DecoratorFileInfo(const QString &filePath);
    explicit DecoratorFileInfo(const QUrl &url);
    explicit DecoratorFileInfo(QSharedPointer<DFMIO::DFileInfo> dfileInfo);
    ~DecoratorFileInfo() = default;

public:
    QSharedPointer<DFMIO::DFileInfo> fileInfoPtr();

    bool isValid() const;
    bool exists() const;
    bool isFile() const;
    bool isDir() const;
    bool isSymLink() const;
    bool isHidden() const;
    bool isWritable() const;

    QUrl url() const;
    QUrl parentUrl() const;
    uint ownerId() const;

    QString suffix() const;
    QString completeSuffix() const;
    QString filePath() const;
    QString parentPath() const;
    QString fileName() const;
    QString symLinkTarget() const;
    quint64 size() const;
    QVariant customAttribute(const char *key, const DFMIO::DFileInfo::DFileAttributeType type);

    QList<QString> standardIconNames() const;

    DFMIO::DFile::Permissions permissions() const;

    bool notifyAttributeChanged();

    DFMIOError lastError() const;

private:
    QSharedPointer<DecoratorFileInfoPrivate> d = nullptr;
};

}

#endif   // DECORATORFILEINFO_H
