/*
 * Copyright (C) 2022 Uniontech Software Technology Co., Ltd.
 *
 * Author:     zhangyu<zhangyub@uniontech.com>
 *
 * Maintainer: zhangyu<zhangyub@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "canvasplugin.h"
#include "canvasmanager.h"
#include "utils/fileutil.h"
#include "canvasdbusinterface.h"

#include "dfm-base/utils/clipboard.h"

#include <QDBusConnection>

using namespace ddplugin_canvas;
DFMBASE_USE_NAMESPACE

void CanvasPlugin::initialize()
{
}

bool CanvasPlugin::start()
{
    // initialize file creator
    DesktopFileCreator::instance();

    // 初始化剪切板
    ClipBoard::instance();

    proxy = new CanvasManager();
    proxy->init();

    registerDBus();

    return true;
}

void CanvasPlugin::stop()
{
    delete proxy;
    proxy = nullptr;
}

void CanvasPlugin::registerDBus()
{
    Q_ASSERT(proxy);
    auto ifs = new CanvasDBusInterface(proxy);

    QDBusConnection conn = QDBusConnection::sessionBus();
    auto registerOptions = QDBusConnection::ExportAllSlots | QDBusConnection::ExportAllSignals | QDBusConnection::ExportAllProperties;
    if (!conn.registerObject("/org/deepin/dde/desktop/canvas", "org.deepin.dde.desktop.canvas", ifs, registerOptions)) {
        qCritical() << "org.deepin.dde.desktop.canvas register object failed" << conn.lastError();
        delete ifs;
    }
}
