/*
 * Copyright (C) 2022 Uniontech Software Technology Co., Ltd.
 *
 * Author:     zhangyu<zhangyub@uniontech.com>
 *
 * Maintainer: zhangyu<zhangyub@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef COLLECTIONMODEL_H
#define COLLECTIONMODEL_H

#include "ddplugin_organizer_global.h"

#include <file/local/localfileinfo.h>

#include <QAbstractProxyModel>

namespace ddplugin_organizer {
class ModelDataHandler;
class CanvasViewShell;
class FileInfoModelShell;
class CollectionModelPrivate;
class CollectionModel : public QAbstractProxyModel
{
    Q_OBJECT
    friend class CollectionModelPrivate;
public:
    explicit CollectionModel(QObject *parent = nullptr);
    ~CollectionModel() override;
    void setModelShell(FileInfoModelShell *shell);
    FileInfoModelShell *modelShell() const;
    void setHandler(ModelDataHandler *);
    ModelDataHandler *handler() const;
    QUrl rootUrl() const;
    QModelIndex rootIndex() const;
    QModelIndex index(const QUrl &url, int column = 0) const;
    DFMLocalFileInfoPointer fileInfo(const QModelIndex &index) const;
    QList<QUrl> files() const;
    QUrl fileUrl(const QModelIndex &index) const;
    void refresh(const QModelIndex &parent, bool global = false, int ms = 50);
    void update();
    bool fetch(const QList<QUrl> &urls);
    bool take(const QList<QUrl> &urls);
public:
    QModelIndex mapToSource(const QModelIndex &proxyIndex) const override;
    QModelIndex mapFromSource(const QModelIndex &sourceIndex) const override;
    QModelIndex index(int row, int column,
                              const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &child) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
    QMimeData *mimeData(const QModelIndexList &indexes) const override;
    bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent) override;
signals:
    void dataReplaced(const QUrl &oldUrl, const QUrl &newUrl);
    void openEditor(const QUrl &url);
private:
    void setSourceModel(QAbstractItemModel *sourceModel) override;
private:
    CollectionModelPrivate *d;
};

}

#endif // COLLECTIONMODEL_H
