/*
 * Copyright (C) 2022 Uniontech Software Technology Co., Ltd.
 *
 * Author:     lixiang<lixianga@uniontech.com>
 *
 * Maintainer: lixiang<lixianga@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "vaultfilewatcher.h"
#include "utils/vaulthelper.h"
#include "private/vaultfilewatcherprivate.h"

#include "dfm-base/utils/universalutils.h"
#include "dfm-base/base/schemefactory.h"

DFMBASE_USE_NAMESPACE
using namespace dfmplugin_vault;
VaultFileWatcher::VaultFileWatcher(const QUrl &url, QObject *parent)
    : AbstractFileWatcher(new VaultFileWatcherPrivate(VaultHelper::vaultToLocalUrl(url), this), parent)
{
    dptr = dynamic_cast<VaultFileWatcherPrivate *>(d.data());
    QUrl localUrl = VaultHelper::vaultToLocalUrl(url);
    dptr->proxyStaging = WatcherFactory::create<AbstractFileWatcher>(localUrl);
    connect(dptr->proxyStaging.data(), &AbstractFileWatcher::fileAttributeChanged,
            this, &VaultFileWatcher::onFileAttributeChanged, Qt::QueuedConnection);
    connect(dptr->proxyStaging.data(), &AbstractFileWatcher::fileDeleted,
            this, &VaultFileWatcher::onFileDeleted, Qt::QueuedConnection);
    connect(dptr->proxyStaging.data(), &AbstractFileWatcher::fileRename,
            this, &VaultFileWatcher::onFileRename, Qt::QueuedConnection);
    connect(dptr->proxyStaging.data(), &AbstractFileWatcher::subfileCreated,
            this, &VaultFileWatcher::onSubfileCreated, Qt::QueuedConnection);
}

VaultFileWatcher::~VaultFileWatcher()
{
    qInfo() << "VaultFileWatcher :" << dptr->proxyStaging->url();
}

void VaultFileWatcher::onFileDeleted(const QUrl &url)
{
    QUrl furl = VaultHelper::instance()->pathToVaultVirtualUrl(url.path());
    emit fileDeleted(furl);
}

void VaultFileWatcher::onFileAttributeChanged(const QUrl &url)
{
    QUrl furl = VaultHelper::instance()->pathToVaultVirtualUrl(url.path());
    emit fileAttributeChanged(furl);
}

void VaultFileWatcher::onFileRename(const QUrl &fromUrl, const QUrl &toUrl)
{
    QUrl furl = VaultHelper::instance()->pathToVaultVirtualUrl(fromUrl.path());
    QUrl turl = VaultHelper::instance()->pathToVaultVirtualUrl(toUrl.path());
    emit fileRename(furl, turl);
}

void VaultFileWatcher::onSubfileCreated(const QUrl &url)
{
    QUrl furl = VaultHelper::instance()->pathToVaultVirtualUrl(url.path());
    qDebug() << url;
    emit subfileCreated(furl);
}
